EXPECTED = {'A': {'extensibility-implied': False,
       'imports': {},
       'object-classes': {},
       'object-sets': {},
       'tags': 'AUTOMATIC',
       'types': {'Type': {'type': 'INTEGER'}},
       'values': {'value': {'type': 'Type', 'value': 5}}},
 'B': {'extensibility-implied': False,
       'imports': {'A': ['Type', 'value']},
       'object-classes': {},
       'object-sets': {},
       'tags': 'AUTOMATIC',
       'types': {'AnotherType': {'type': 'Type'}},
       'values': {'anotherValue': {'type': 'AnotherType', 'value': 'value'}}},
 'C': {'extensibility-implied': False,
       'imports': {'B': ['Type', 'anotherValue']},
       'object-classes': {},
       'object-sets': {},
       'tags': 'AUTOMATIC',
       'types': {'AndAnotherType': {'type': 'Type'}},
       'values': {'andAnotherValue': {'type': 'AndAnotherType',
                                      'value': 'anotherValue'}}},
 'D': {'extensibility-implied': False,
       'imports': {'B': ['Type', 'anotherValue']},
       'object-classes': {},
       'object-sets': {},
       'tags': 'AUTOMATIC',
       'types': {'AndAnotherType': {'type': 'Type'}},
       'values': {'andAnotherValue': {'type': 'AndAnotherType',
                                      'value': 'anotherValue'}}}}